/* Starsiege Quickchat Generator v3 (SQCGen)
 *
 * See SQCGen.html for details.
 *
 *  v1: 31 March 1999 - Drag & Drop to use any 9 quickchats
 *  v2: 14 April 1999 - 81+ quickchats via Shift F1-9 + Ctrl F1-9
 *  v3: 10 June 1999 - unlimited quickchats via Fn or Numpad keymaps,
 *      print-friendly output, SQCFilter
 *
 * Created by Pincushion (pgarpincushion@geocities.com)
 */
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <windows.h>

static char appname[] = "SQCGen v3";
static char helpfn[] = "SQCGen.html";
static char csfn[] = "SQCGen.cs";
static char fmapfn[] = "MapFKeys";
static char nmapfn[] = "MapNumpad";
static char outfn[] = "scripts\\quickchat.cs";
static char buf[2048], savbuf[2048];
static char sqcgendir[2048];
static char ssdir[2048];

static void usage( char *pgm )
{
	char *p;

	p = strrchr( pgm, '\\' );
	if ( p ) pgm = p + 1;
	fprintf( stderr,
"Usage: %s <file>\n", pgm );
	fprintf( stderr,
"Generates %s from <file>.  See %s for help.\n", outfn, helpfn );
}

int main( int argc, char **argv )
{
	FILE *ifp, *ofp, *ffp, *nfp;
	char *fn, *p, *txt, *wav, *mapext;
	int argi, ret, nqc, id, xid, pow, dig, verbose=0, pause=0, htmlout=0;

	ret = -1;
	for ( argi=1 ; argi<argc ; argi++ ) {
		if ( argi == argc ) {
usage_err:
			usage( argv[0] );
			goto done;
		}
		if ( argv[argi][0] != '-' )
			break;
		if ( strcmp( argv[argi], "-v" ) == 0 )
			verbose = 1;
		else if ( strcmp( argv[argi], "-html" ) == 0 )
			htmlout = 1;
		else
			goto usage_err;
	}
	mapext = htmlout ? "html" : "txt";

	/* Infer the Starsiege folder based on where our executable lives */
	strcpy( sqcgendir, argv[0] );
	p = strrchr( sqcgendir, '\\' );
	if ( p == 0 ) {
where:
		/* guess where we are */
		strcpy( sqcgendir, "." );
		strcpy( ssdir, ".." );
	} else {
		*p = '\0';
		strcpy( ssdir, sqcgendir );
		p = strrchr( ssdir, '\\' );
		if ( p == 0 )
			goto where;
		*p = '\0';
	}

	/* Create output files */
	fn = buf;
	sprintf( fn, "%s\\%s", ssdir, outfn );
	if ( (ofp = fopen( fn, "w" )) == 0 ) {
openerr:
		fprintf( stderr, "%s: Can't open %s.\n", argv[0], fn );
		goto done;
	}
	sprintf( fn, "%s\\%s.%s", sqcgendir, fmapfn, mapext );
	if ( (ffp = fopen( fn, "w" )) == 0 )
		goto openerr;
	sprintf( fn, "%s\\%s.%s", sqcgendir, nmapfn, mapext );
	if ( (nfp = fopen( fn, "w" )) == 0 )
		goto openerr;
	fprintf( ofp, "// Generated from %s by %s\n", argv[argi], appname );
	fprintf( ofp, "\n" );
	if ( htmlout ) {
		fprintf( ffp, "<HTML><BODY>\n" );
		fprintf( nfp, "<HTML><BODY>\n" );
	}
	fprintf( ffp, "// Generated from %s by %s\n", argv[argi], appname );
	if ( htmlout ) fprintf( ffp, "<BR>" );
	fprintf( ffp, "// \"321:\" means Shift-F3 Shift-F2 Ctrl-F1\n\n" );
	fprintf( nfp, "// Generated from %s by %s\n", argv[argi], appname );
	if ( htmlout ) fprintf( nfp, "<BR>" );
	fprintf( nfp, "// \"321:\" means Numpad3 Numpad2 Numpad1 NumpadEnter\n\n" );
	if ( htmlout ) {
		fprintf( ffp, "<MULTICOL COLS=5>\n" );
		fprintf( ffp, "<FONT SIZE=-2>\n" );
		fprintf( nfp, "<MULTICOL COLS=5>\n" );
		fprintf( nfp, "<FONT SIZE=-2>\n" );
	}

	/* .cs file */
	sprintf( fn, "%s\\%s", sqcgendir, csfn );
	if ( (ifp = fopen( fn, "r" )) == 0 )
		goto openerr;
	if ( verbose )
		fprintf( stdout, "Header Config:\n" );
	for ( ; ; ) {
		fgets( buf, sizeof(buf)-1, ifp );
		if ( feof( ifp ) )
			break;
		fputs( buf, ofp );
		if ( verbose )
			fputs( buf, stdout );
	}
	fclose( ifp );

	/* messages */
	fn = argv[argi];
	if ( (ifp = fopen( fn, "r" )) == 0 )
		goto openerr;
	if ( verbose )
		fprintf( stdout, "Quickchats:\n" );
	for ( id=nqc=0 ; ; ) {
		fgets( buf, sizeof(buf)-1, ifp );
		if ( feof( ifp ) )
			break;
		buf[sizeof(buf)-1] = '\0';
		for ( p=buf ; ; p++ ) {
			if ( *p == '\0' )
				goto next; /* all white space */
			if ( !isspace(*p) )
				break;
		}
		p = strstr( buf, "id=" );
		if ( p ) {
			id = atoi( p+3 );
			continue;
		}
		p = strstr( buf, "fn=" );
		if ( p ) {
			for ( id=0, p+=3 ; isdigit(*p) ; p++ )
				id = 9 * id + (*p - '1');
			continue;
		}
		if ( buf[0] == '/' ) {
			if ( htmlout ) {
				fprintf( ffp, "<BR>" );
				fprintf( nfp, "<BR>" );
			}
			fputs( buf, ffp );
			fputs( buf, nfp );
			continue; /* comment */
		}
		if ( buf[0] == '.' )
			goto skip; /* placeholder, skip ID */
		p = strstr( buf, "Say" );
		strcpy( savbuf, buf );
		if ( p ) {
			/* parse start of a Say statement */
			p = strchr( p, '\"' ); /* 1st quote of who */
			if ( p == 0 ) {
syntax_err:
				pause = 1;
				fprintf( stderr, "Skipped garble: %s\n", savbuf );
				continue;
			}
			p = strchr( ++p, '\"' ); /* 2nd quote of who */
			if ( p == 0 ) goto syntax_err;
			++p;
		} else
			p = buf;
		/* parse rest of Say statement or a line of only "txt" "wav" */
		p = strchr( p, '\"' ); /* 1st quote of txt */
		if ( p == 0 ) goto syntax_err;
		txt = ++p;
		p = strchr( p, '\"' ); /* 2nd quote of txt */
		if ( p == 0 ) goto syntax_err;
		*p++ = '\0';
		p = strchr( p, '\"' ); /* 1st quote of wav */
		if ( p == 0 ) goto syntax_err;
		wav = ++p;
		p = strchr( p, '\"' ); /* 2nd quote of wav */
		if ( p == 0 ) goto syntax_err;
		*p = '\0';
		fprintf( ofp, "$qcT[%d]=\"%s\"; $qcW[%d]=\"%s\";\n", id, txt, id, wav );
		if ( htmlout ) {
			fprintf( ffp, "<BR>" );
			fprintf( nfp, "<BR>" );
		}
		for ( xid=id, dig=0, pow=9*9*9*9*9*9*9 ; pow ; pow/=9 )
			if ( ((xid == 0) && (pow == 1)) || dig || (xid/pow) ) {
				dig = 1;
				fprintf( ffp, "%d", (xid/pow)+1 );
				xid -= (xid/pow) * pow;
			}
		fprintf( ffp, ": %s\n", txt );
		fprintf( nfp, "%d", id );
		fprintf( nfp, ": %s\n", txt );
		if ( verbose )
			fprintf( stdout, "\"%s\" %s\n", txt, wav );
		++nqc;
skip:
		++id;
next:
		;
	}
	if ( htmlout ) {
		fprintf( ffp, "</FONT></MULTICOL></BODY></HTML>\n" );
		fprintf( nfp, "</FONT></MULTICOL></BODY></HTML>\n" );
	}
	fclose( ifp );
	fclose( ofp );
	fclose( ffp );
	fclose( nfp );
	printf( "Created %s, %s.%s, %s.%s.\n%d quickchats.\n",
		outfn, fmapfn, mapext, nmapfn, mapext, nqc );
	ret = 0;

done:
	fflush( stdout );
	fflush( stderr );
	Sleep( ((ret<0) || pause) ? 5000 : 1000 );
	return( ret );
}
